/*  aminoacids.h
 *
 *  Copyright (C) 2012, Institute of Enzymology,
 *  Daniel Kozma,  kozmad at enzim dot hu

 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __aminoacids_h_
#define __aminoacids_h_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/******************** Amino acids *********************/
/*******************************************************
 NAME          3LET   1LET  POLARITY CHARGE  HIDROPATHY
Alanine		Ala	A	a	00	-0.11
Arginine	Arg	R	p	+1	-2.58
Asparagine	Asn	N	p	00	−2.05
Asparticacid	Asp	D	p	-1	−3.49
Asn/Asp		Asx	B	p	??	+0.12
Cysteine	Cys	C	a	00	+0.13
Glutamicacid	Glu	E	p	-1	−2.68
Glutamine	Gln	Q	p	00	−2.36
Gln/Glu		Glx	Z	p	??	-2.52
Glycine		Gly	G	a	00	−0.74
Histidine	His	H	p	00	−2.06
Isoleucine	Ile	I	a	00	+0.6
Leucine		Leu	L	a	00	+0.55
Leu/Ile		Xle	J	a	00	?????
Lysine		Lys	K	p	+1	−2.71
Methionine	Met	M	a	00	+0.10
Phenylalanine	Phe	F	a	00	+0.32
Proline		Pro	P	a	00	−2.23
Serine		Ser	S	p	00	−0.84
Threonine	Thr	T	p	00	−0.52
Tryptophan	Trp	W	a	00	−0.30
Tyrosine	Tyr	Y	p	00	−0.68
Unknown		Xaa	X	?	??	?????
Valine		Val	V	a	00	+0.31
*********************************************************/

//static const char* aa3c[] = {"ALA", "ARG", "ASN", "ASP", "CYS", "GLU", "GLN", "GLY", "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER", "THR", "TRP", "TYR", "VAL", "*"};
//static const char aa1c[] = "ARNDCEQGHILKMFPSTWYV*";

#define AA_NUM  27
#define STD_AA_NUM 20
#define PSEUDO_AA_NUM 7
#define AA_INDEX(c) ( (int)c<65 || (int)c>90 )?(AA_NUM-1):((int)c-65)


#define FOR_STD_AA(x)  for (int I_AA=0, x=0; I_AA<STD_AA_NUM; I_AA++, x=(int)STD_AA[I_AA]-65) 
#define FOR_ALPHABET(ABC, x)  for (int I_AA=0, x=0; I_AA<strlen(ABC); I_AA++, x=AA_INDEX(ABC[I_AA])) 
#define FOR_ALL_AA(x)  for (x=0; x<AA_NUM; x++)

// acidic (D, E)
// basic (K, R, H)
// aromatic (F, Y, W)
// aliphatic (A, G, I, L, V, M)
// amidic (N, Q)
// hydroxylated (S, T, Y)
// cysteine (C)
// proline (P)

//                            A  B  C  D  E  F  G  H  I  J  K  L  M   N   O  P   Q   R   S   T   U  V  W   X  Y   Z   -
static const int AA_TYPE[] = {0, 1, 2, 3, 3, 5, 0, 7, 0, 9, 7, 0, 0, 13, 14, 15, 13, 7, 18, 18, 20, 0, 5, 23, 5, 25, 26};
static const char A[]="ABCDEFGHIJKLMNOPQRSTUVWXYZ-";
static const char AAA[][4]={"ALA", "ASX", "CYS", "ASP", "GLU", "PHE", "GLY", "HIS", "ILE", "XLE", "LYS", "LEU", "MET", "ASN", "PYL", "PRO", "GLN", "ARG", "SER", "THR", "SEC", "VAL", "TRP", "UNK", "TYR", "GLX", "---"};
static const char STD_AA[]="ACDEFGHIKLMNPQRSTVWY";
static const char PSEUDO_AA[]="BJOUXZ-";

// from the Ranganathan lab for SCA
static const double SCA_background[] =
/*  A        B      C        D         E           F        G          H       I       J       K       L          M        N       O      P          Q         R         S          T     U       V         W      X       Y      Z    - */
{0.072658, 0.0, 0.024692, 0.050007, 0.061087, 0.041774, 0.071589, 0.023392, 0.052691, 0.0, 0.063923,0.089093, 0.023150, 0.042931, 0.0, 0.052228, 0.039871, 0.052012, 0.073087, 0.055606, 0.0, 0.063321, 0.012720, 0.0, 0.032955, 0.0, 0.0};

static const char HYD[]=
/*   A      B      C      D      E      F      G      H      I      J      K      L      M      N      O      P      Q      R      S      T      U      V     W      X      Y      Z      -   */
{ -0.11, +0.12, -0.13, -3.49, -2.68, +0.32, -0.74, -2.06, +0.60, +0.00, -2.71, +0.55, +0.10, -2.05, +0.00, -2.23, -2.36, -2.58, -0.84, -0.52, +0.00, +0.31, -0.30, +0.00, -0.68, -2.52, +0.00, };


static const double ASA[]={104,128,149,181,218, 71,188,175,200,181,199,160,
			150,180,236,119,140,151,260,238,
			100,100};

static const char NON_STD_AA[3][2][4] = 
                                 { {"MEN", "ASN"},
                                   {"CSH", "CYS"},
			           {"MSE", "MET"} }; 

static const int BLOSUM62[AA_NUM][AA_NUM] = {
/*      A,  B,  C,  D,  E,  F,  G,  H,  I,  J,  K,  L,  M,  N,  O,  P,  Q,  R,  S,  T,  U,  V,  W,  X,  Y,  Z,  - */
/*A*/ { 4, -2,  0, -2, -1, -2,  0, -2, -1, -1, -1, -1, -1, -2, -4, -1, -1, -1,  1,  0, -4,  0, -3, -1, -2, -1, -4},
/*B*/ {-2,  4, -3,  4,  1, -3, -1,  0, -3, -3,  0, -4, -3,  4, -4, -2,  0, -1,  0, -1, -4, -3, -4, -1, -3,  0, -4},
/*C*/ { 0, -3,  9, -3, -4, -2, -3, -3, -1, -1, -3, -1, -1, -3, -4, -3, -3, -3, -1, -1, -4, -1, -2, -1, -2, -3, -4},
/*D*/ {-2,  4, -3,  6,  2, -3, -1, -1, -3, -3, -1, -4, -3,  1, -4, -1,  0, -2,  0, -1, -4, -3, -4, -1, -3,  1, -4},
/*E*/ {-1,  1, -4,  2,  5, -3, -2,  0, -3, -3,  1, -3, -2,  0, -4, -1,  2,  0,  0, -1, -4, -2, -3, -1, -2,  4, -4},
/*F*/ {-2, -3, -2, -3, -3,  6, -3, -1,  0,  0, -3,  0,  0, -3, -4, -4, -3, -3, -2, -2, -4, -1,  1, -1,  3, -3, -4},
/*G*/ { 0, -1, -3, -1, -2, -3,  6, -2, -4, -4, -2, -4, -3,  0, -4, -2, -2, -2,  0, -2, -4, -3, -2, -1, -3, -2, -4},
/*H*/ {-2,  0, -3, -1,  0, -1, -2,  8, -3, -3, -1, -3, -2,  1, -4, -2,  0,  0, -1, -2, -4, -3, -2, -1,  2,  0, -4},
/*I*/ {-1, -3, -1, -3, -3,  0, -4, -3,  4,  3, -3,  2,  1, -3, -4, -3, -3, -3, -2, -1, -4,  3, -3, -1, -1, -3, -4},
/*J*/ {-1, -3, -1, -3, -3,  0, -4, -3,  3,  3, -3,  3,  2, -3, -4, -3, -2, -2, -2, -1, -4,  2, -2, -1, -1, -3, -4},
/*K*/ {-1,  0, -3, -1,  1, -3, -2, -1, -3, -3,  5, -2, -1,  0, -4, -1,  1,  2,  0, -1, -4, -2, -3, -1, -2,  1, -4},
/*L*/ {-1, -4, -1, -4, -3,  0, -4, -3,  2,  3, -2,  4,  2, -3, -4, -3, -2, -2, -2, -1, -4,  1, -2, -1, -1, -3, -4},
/*M*/ {-1, -3, -1, -3, -2,  0, -3, -2,  1,  2, -1,  2,  5, -2, -4, -2,  0, -1, -1, -1, -4,  1, -1, -1, -1, -1, -4},
/*N*/ {-2,  4, -3,  1,  0, -3,  0,  1, -3, -3,  0, -3, -2,  6, -4, -2,  0,  0,  1,  0, -4, -3, -4, -1, -2,  0, -4},
/*O*/ {-4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4,  1, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4},
/*P*/ {-1, -2, -3, -1, -1, -4, -2, -2, -3, -3, -1, -3, -2, -2, -4,  7, -1, -2, -1, -1, -4, -2, -4, -1, -3, -1, -4},
/*Q*/ {-1,  0, -3,  0,  2, -3, -2,  0, -3, -2,  1, -2,  0,  0, -4, -1,  5,  1,  0, -1, -4, -2, -2, -1, -1,  4, -4},
/*R*/ {-1, -1, -3, -2,  0, -3, -2,  0, -3, -2,  2, -2, -1,  0, -4, -2,  1,  5, -1, -1, -4, -3, -3, -1, -2,  0, -4},
/*S*/ { 1,  0, -1,  0,  0, -2,  0, -1, -2, -2,  0, -2, -1,  1, -4, -1,  0, -1,  4,  1, -4, -2, -3, -1, -2,  0, -4},
/*T*/ { 0, -1, -1, -1, -1, -2, -2, -2, -1, -1, -1, -1, -1,  0, -4, -1, -1, -1,  1,  5, -4,  0, -2, -1, -2, -1, -4},
/*U*/ {-4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4,  1, -4, -4, -4, -4, -4, -4},
/*V*/ { 0, -3, -1, -3, -2, -1, -3, -3,  3,  2, -2,  1,  1, -3, -4, -2, -2, -3, -2,  0, -4,  4, -3, -1, -1, -2, -4},
/*W*/ {-3, -4, -2, -4, -3,  1, -2, -2, -3, -2, -3, -2, -1, -4, -4, -4, -2, -3, -3, -2, -4, -3, 11, -1,  2, -2, -4},
/*X*/ {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -4, -1, -1, -1, -1, -1, -4, -1, -1, -1, -1, -1, -4},
/*Y*/ {-2, -3, -2, -3, -2,  3, -3,  2, -1, -1, -2, -1, -1, -2, -4, -3, -1, -2, -2, -2, -4, -1,  2, -1,  7, -2, -4},
/*Z*/ {-1,  0, -3,  1,  4, -3, -2,  0, -3, -3,  1, -3, -1,  0, -4, -1,  4,  0,  0, -1, -4, -2, -2, -1, -2,  4, -4},
/*-*/ {-4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4,  1}
};

const char getAAChar(char * str);
const char *getAAName(char c);


#endif /* aminoacids.h */


