/*  corrmut.c
 *
 *  Copyright (C) 2012, Institute of Enzymology,
 *  Daniel Kozma,  kozmad at enzim dot hu

 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**********************************************************
  A utility for calculating multiple correlation on MSA 
 to find correlated mutations.                         
                                                        
  USAGE:
    cm = createNewCorrmut();
    loadMsaFromFile(cm, filename)
    calculateCovarianceMatrix(cm, METHOD_TYPE);
    printCovarianceMatrix(cm);

**********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_sf_gamma.h>
#include "nonstdio.h"
#include "matrix.h"
#include "aminoacids.h"
#include "corrmut.h"

extern int N_CONTACTS;
extern int SEQSEP;
extern int FIRST_N;
extern double UPPER_THRESHOLD;
extern double LOWER_THRESHOLD;
extern double UPPER_SIGNIFICANCE;
extern double LOWER_SIGNIFICANCE;


corrmut* createNewCorrmut() {
	corrmut* cm = (corrmut*) malloc (sizeof(corrmut));
	cm->n = 0;
	cm->l = 0;
	cm->msa = NULL;
	cm->cons = NULL;
	cm->probI = NULL;
	cm->probIJ = NULL;
	cm->probAA = NULL;
	cm->msaMatrix = NULL;
	cm->msaMatrixMean = NULL;
	cm->msaMatrixStddev = NULL;
	cm->msaSimMatrix = NULL;
	cm->covMatrix = NULL;
	cm->signMatrix = NULL;
	return cm;
}

void calculateCovarianceMatrix(corrmut* cm, int type) {
        int i,j;
	
        if (cm->l==0) return;
	cm->covMatrix = createPRECISIONMatrix(cm->l, cm->l);
        for (i=0; i<cm->l; i++) for (j=i; j<cm->l; j++) cm->covMatrix[i][j] = cm->covMatrix[j][i] = 0.0;
	switch (type) {
		case GOBEL:
		case McBASC:
			calculateCovarianceMatrixGOBEL(cm);
			break;
		case OMES:
			calculateCovarianceMatrixOMES(cm);
			break;
		case ELSC:
			calculateCovarianceMatrixELSC(cm);
			break;
		case SCA:
			calculateCovarianceMatrixSCA(cm);
			break;
		case MI:
			calculateCovarianceMatrixMI(cm);
			break;
		case UC:
			calculateCovarianceMatrixUC(cm);
			break;
		case RANDOM:
			randomPrediction(cm);
			break;
		case NO_CORRMUT:
		default:
			break;
	}
	return;
}

void calculateCovarianceMatrixGOBEL(corrmut* cm) {
	int i,j,k,l;
	PRECISION N;

	createMatrixFromMSA(cm);
	calculateProbability(cm, PROB_I);
	calculateProbability(cm, PROB_IJ);
	calculateConservation(cm, QUAD);
	calculateMsaMatrixValues(cm);
	calculateMsaSimMatrix(cm);
	reduceMsaMatrixValues(cm);

	N = (PRECISION)(cm->n*(cm->n-1)/2.-1.0);
	for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) {
		cm->covMatrix[i][j] = 0.0;
		for (k=0; k<cm->n; k++) for (l=0; l<k; l++)
			cm->covMatrix[i][j] += /*cm->msaSimMatrix[k][l]**/cm->msaMatrix[i][k][l]*cm->msaMatrix[j][k][l];
		cm->covMatrix[j][i] = (cm->covMatrix[i][j] /= N);
	}
	return;
}

void calculateCovarianceMatrixOMES(corrmut* cm) {
	int i,j,k,l,n;


	calculateProbability(cm, PROB_I_F);
	calculateProbability(cm, PROB_IJ_F);
	calculateConservation(cm, QUAD);

	if (cm->signMatrix==NULL) cm->signMatrix = createPRECISIONMatrix(cm->l, cm->l);
	for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) cm->signMatrix[i][j] = cm->signMatrix[j][i] = -9999.9999;
	for (i=0; i<cm->l-1; i++) if (cm->probI[i][AA_NUM-1]<=GAP_THRESHOLD) for (j=i+1; j<cm->l; j++) if (cm->probI[j][AA_NUM-1]<=GAP_THRESHOLD) {
		n = -1;
		cm->covMatrix[i][j] = 0.0;
		FOR_ALL_AA(k) if (cm->probI[i][k]!=0.) FOR_ALL_AA(l) if (cm->probI[j][l]!=0.) {
			cm->covMatrix[i][j] += pow(cm->probIJ[i][j][k][l]-cm->probI[i][k]*cm->probI[j][l], 2.)/(cm->probI[i][k]*cm->probI[j][l]);
			n++;
		}
		cm->covMatrix[j][i] = (cm->covMatrix[i][j] *= cm->n);
		if (n==0) {
			cm->signMatrix[j][i] = cm->signMatrix[i][j] = -9999.9999;
			continue;
		}
		cm->signMatrix[j][i] = cm->signMatrix[i][j] = gsl_cdf_chisq_Q(cm->covMatrix[i][j], n);
	}
	return;
}

void calculateCovarianceMatrixMI(corrmut* cm) {
	int i,j;

	calculateProbability(cm, PROB_I_F);
	calculateProbability(cm, PROB_IJ_F);
	calculateConservation(cm, ENTROPY);

	for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) {
		cm->covMatrix[i][j] = 0.0;
		FOR_ALPHABET( ALPHABET, k ) if (cm->probI[i][k]>0.0) FOR_ALPHABET( ALPHABET, l ) if (cm->probI[j][l]>0.0)
			if (cm->probIJ[i][j][k][l] > 0.0) cm->covMatrix[i][j] -= cm->probIJ[i][j][k][l]*log(cm->probIJ[i][j][k][l]);
		cm->covMatrix[j][i] = (cm->covMatrix[i][j] = (cm->cons[i]+cm->cons[j]-cm->covMatrix[i][j]));
	}
	return;
}

void calculateCovarianceMatrixUC(corrmut* cm) {
	int i,j;

	calculateProbability(cm, PROB_I_F);
	calculateProbability(cm, PROB_IJ_F);
	calculateConservation(cm, ENTROPY);

	for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) {
		cm->covMatrix[i][j] = 0.0;
		FOR_ALPHABET( ALPHABET, k ) if (cm->probI[i][k]>0.0) FOR_ALPHABET( ALPHABET, l ) if (cm->probI[j][l]>0.0)
			if (cm->probIJ[i][j][k][l] > 0.0) cm->covMatrix[i][j] -= cm->probIJ[i][j][k][l]*log(cm->probIJ[i][j][k][l]);
		cm->covMatrix[j][i] = (cm->covMatrix[i][j] = 2.*(cm->cons[i]+cm->cons[j]-cm->covMatrix[i][j])/(cm->cons[i]+cm->cons[j])); 
	}
	return;
}
void calculateCovarianceMatrixELSC(corrmut* cm) {
	int i,j,k,max;
	double dmax;
	double score;
	corrmut *tmp;
        
        tmp = createNewCorrmut();
        tmp->l = cm->l;        

	calculateProbability(cm, PROB_I_F);

        
	for (i=0; i<cm->l-1; i++) {
                dmax = 0.0;
                max = 0;
                FOR_ALPHABET( ALPHABET, j )
			if(dmax<cm->probI[i][j]) {
				dmax = cm->probI[i][j];
				max = j;
			}
		
                if (dmax==0.0) continue;

		tmp->n = (int)(cm->probI[i][max]*cm->n+0.5);
		tmp->msa = (char **) malloc (tmp->n*sizeof(char *));

		for (k=0, j=0; j<cm->n; j++)
			if (cm->msa[j][i]==A[max])
				tmp->msa[k++] = cm->msa[j];

		calculateProbability(tmp, PROB_I_F);


                
                double **ideal = createDoubleMatrix(tmp->l, AA_NUM);
                for (j=0; j<tmp->l; j++) {
                        int sum = 0;
			double factn = 0.0;
			double factN = 0.0;
                        double r[AA_NUM];

			FOR_ALPHABET( ALPHABET, k ) {
				factn += tmp->probI[j][k];
				factN += cm->probI[j][k];
			}

                        FOR_ALPHABET( ALPHABET, k ) {
                                r[k] = 0.0;
				ideal[j][k] = 0.0;
                                if (factN>0.0) ideal[j][k] = ((float)(int)(tmp->n*factn+0.5))*((float)(int)(cm->probI[j][k]*cm->n+0.5))/((double)((int)(cm->n*factN+0.5)));
                                r[k] = ideal[j][k]-(int)(ideal[j][k]);
                                sum += (int)(ideal[j][k]);
                        }


                        for (k=0; k<((int)(tmp->n*factn+0.5)-sum); k++) {
				dmax = -1.0;
                                max = 0;
                                FOR_ALPHABET( ALPHABET, l )
                                        if(dmax<=r[l]) {
						dmax = r[l];
                                                max = l;
                                        }
                                ideal[j][max] += 1.0;

                                r[max] = -1.0;
                        }

                        score = 0.0;
                        FOR_ALPHABET( ALPHABET, k ) {
//printf("%d %d %d\n", (int)(cm->probI[j][k]*cm->n+0.5), (int)(tmp->n*tmp->probI[j][k]+0.5), (int)(ideal[j][k]));
                                score-=gsl_sf_lnchoose((int)(cm->probI[j][k]*cm->n+0.5), (int)(tmp->n*tmp->probI[j][k]+0.5));
                                if ( (int)(cm->probI[j][k]*cm->n+0.5) > (int)(ideal[j][k]) )
                                        score+=gsl_sf_lnchoose( (int)(cm->probI[j][k]*cm->n+0.5), (int)(ideal[j][k]) );
                        }

                        cm->covMatrix[i][j] = score;
                }
                freePRECISIONMatrix(ideal, tmp->l);
                if (tmp->probI!=NULL) freePRECISIONMatrix(tmp->probI, tmp->l);
                if (tmp->msa!=NULL) free(tmp->msa);
                tmp->probI=NULL;
                tmp->msa=NULL;
	}
        freeCorrmut(tmp);
        return;
}

void calculateCovarianceMatrixSCA(corrmut* cm) {
        int i,j,k,max;
        double dmax, t;
        corrmut *tmp;
        
        tmp = createNewCorrmut();
        tmp->l = cm->l;        

        calculateProbability(cm, PROB_I_F);
        double **fullLocklessScore = createPRECISIONMatrix(cm->l, AA_NUM);
        for (i=0; i<cm->l; i++) {
                FOR_ALPHABET(ALPHABET, k) fullLocklessScore[i][k] = 0.0;
                FOR_ALPHABET(ALPHABET, k) {
                        t = gsl_sf_lngamma(100+1.) - gsl_sf_lngamma(100*cm->probI[i][k]+1.) - gsl_sf_lngamma(100-100*cm->probI[i][k]+1.);
                        t += 100*cm->probI[i][k]*log(SCA_background[k]);
                        t += (100-100*cm->probI[i][k])*log(1.-SCA_background[k]);
                        fullLocklessScore[i][k] = t;
                }
        }       

        
        for (i=0; i<cm->l; i++) {
                dmax = 0.0;
                max = 0;
                FOR_ALPHABET(ALPHABET, j) if(A[j]!='-')
                        if(dmax<cm->probI[i][j]) {
                                dmax = cm->probI[i][j];
                                max = j;
                        }
                if (dmax==0.0) continue;

                tmp->n = (int)(cm->probI[i][max]*cm->n+0.5);
                tmp->msa = (char **) malloc (tmp->n*sizeof(char *));

                for (k=0, j=0; j<cm->n; j++)
                        if (cm->msa[j][i]==A[max])
                                tmp->msa[k++] = cm->msa[j];

                calculateProbability(tmp, PROB_I_F);
                
                for (j=0; j<tmp->l; j++) {
                        double score = 0.0;
                        FOR_ALPHABET(ALPHABET, k) {
                               t = gsl_sf_lngamma(100+1.) - gsl_sf_lngamma(100*tmp->probI[j][k]+1.) - gsl_sf_lngamma(100-100*tmp->probI[j][k]+1.);
                               t += 100*tmp->probI[j][k]*log(SCA_background[k]);
                               t += (100-100*tmp->probI[j][k])*log(1.-SCA_background[k]);
                               t -= fullLocklessScore[j][k];
                               score += t*t;
                        }
                        score = sqrt(score);
                        cm->covMatrix[i][j] = score;
                }
                
                if (tmp->probI!=NULL) freePRECISIONMatrix(tmp->probI, tmp->l);
                if (tmp->msa!=NULL) free(tmp->msa);
                tmp->probI=NULL;
                tmp->msa=NULL;
        }
        freePRECISIONMatrix(fullLocklessScore, tmp->l);
        freeCorrmut(tmp);
        return;
}

void calculateNthOrderCovariance(corrmut *cm) {
        int i,j,k,l,m;
        PRECISION sum;
        PRECISION N;

        calculateProbability(cm, PROB_I);
        calculateProbability(cm, PROB_IJ);
        calculateConservation(cm, QUAD);
        calculateMsaMatrixValues(cm);
        calculateMsaSimMatrix(cm);
        reduceMsaMatrixValues(cm);

        /*TODO tetszoleges rendu correlacio vizsgalata*/
        N = (PRECISION)(cm->n*(cm->n-1)/2.-1.0);
        for (i=0; i<cm->l; i++) for (j=0; j<i; j++) for (m=0; m<j; m++) {
                sum = 0.0;
                for (k=0; k<cm->n; k++) for (l=0; l<k; l++)
                        sum += cm->msaSimMatrix[k][l]*cm->msaMatrix[i][k][l]*cm->msaMatrix[j][k][l]*cm->msaMatrix[m][k][l];
                printf("%d %d %d %f\n", i, j, m, sum/N);
        }
        return;
}

void randomPrediction(corrmut* cm) {
	int t=0;
	int i, j, n;
	gsl_rng * r;

	if (cm->l==0) return;

	gsl_rng_env_setup();
	r = gsl_rng_alloc (gsl_rng_default);

	// generating random contacts
	if (N_CONTACTS>(cm->l*(cm->l-1)/2)) {
		printf("CORRMUT ::: N_CONTACTS is greater than the number of possible contacts!\n");
		return;
	}
	for (n=0; n<N_CONTACTS; ) {
		t++;
		i = (int)(cm->l*gsl_rng_uniform(r));
		j = (int)(cm->l*gsl_rng_uniform(r));
		if (abs(j-i)<SEQSEP) continue;
		if (t>1000) {
			printf("CORRMUT ::: number of tries exceeds 1000, stop generating random contacts!\n");
			break;
		}
		if (cm->covMatrix[i][j]!=0.0) continue;
		cm->covMatrix[j][i] = cm->covMatrix[i][j] = 1.0;
		n++;
		t=0;
	}
	gsl_rng_free (r);
}

void evaluatePrediction(corrmut* cm, int type) {
	int i,j,k,n, N;
	double **res = NULL;
	
	if (cm->l==0) {
		printf("EVALUATE ::: Covariance matrix size is zero!\n");
		return;
	}
	if (cm->covMatrix==NULL) {
		printf("EVALUATE ::: Covariance matrix was not defined!\n");
		return;
	}

	switch (type) {
		case FIRST_N_LARGEST:
			n = cm->l*(cm->l-1)/2.;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			sort(res, 2, DESCENDING);
			
			for (i=0; i<cm->l; i++) for(j=0; j<=i; j++)
				cm->covMatrix[i][j] = cm->covMatrix[j][i] = 0.0;
			
			for (k=0; k<FIRST_N; k++) {
				i = (int)(res[k][0]+0.5);
				j = (int)(res[k][1]+0.5);
				if (abs(i-j)<SEQSEP) continue;
				cm->covMatrix[i][j] = cm->covMatrix[j][i] = 1.0;
			}

			for (i=0; res[i]!=NULL; i++) free(res[i]);
			free(res);
			break;
		case FIRST_N_SMALLEST:
			n = cm->l*(cm->l-1)/2.;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			sort(res, 2, ASCENDING);
			
			for (i=0; i<cm->l; i++) for(j=0; j<=i; j++)
				cm->covMatrix[i][j] = cm->covMatrix[j][i] = 0.0;
			
			for (k=0; k<FIRST_N; k++) {
				i = (int)(res[k][0]+0.5);
				j = (int)(res[k][1]+0.5);
				if (abs(i-j)<SEQSEP) continue;
				cm->covMatrix[i][j] = cm->covMatrix[j][i] = 1.0;
			}

			for (i=0; res[i]!=NULL; i++) free(res[i]);
			free(res);
			break;
			
		case FIRST_N_LARGEST_WITH_SIGNIFICANCE_INTERVAL:
			N = 0;
			n = cm->l*(cm->l-1)/2.;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			sort(res, 2, DESCENDING);
			
			for (i=0; i<cm->l; i++) for(j=0; j<=i; j++)
				cm->covMatrix[i][j] = cm->covMatrix[j][i] = 0.0;
			
			for (k=0; res[k]!=NULL && N<FIRST_N; k++) {
				i = (int)(res[k][0]+0.5);
				j = (int)(res[k][1]+0.5);
				if (abs(i-j)<SEQSEP) continue;
				if (cm->signMatrix[i][j]<=UPPER_SIGNIFICANCE && cm->signMatrix[i][j]>=LOWER_SIGNIFICANCE) {
					cm->covMatrix[i][j] = cm->covMatrix[j][i] = 1.0;
					N++;
				}
			}

			for (i=0; res[i]!=NULL; i++) free(res[i]);
			free(res);
			break;			
			
			
		case GREATER_THAN_THRESHOLD:
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++) {
				cm->covMatrix[i][j] = (cm->covMatrix[i][j]>UPPER_THRESHOLD && abs(i-j)>=SEQSEP)?1.0:0.0;
			}
			break;
		case LESS_THAN_THRESHOLD:
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++)
				cm->covMatrix[i][j] = (cm->covMatrix[i][j]<LOWER_THRESHOLD && abs(i-j)>=SEQSEP)?1.0:0.0;
			break;
		case THRESHOLD_INTERVAL:
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++)
				cm->covMatrix[i][j] = (cm->covMatrix[i][j]>=LOWER_THRESHOLD && cm->covMatrix[i][j]<=UPPER_THRESHOLD && abs(i-j)>=SEQSEP)?1.0:0.0;
			break;
		case GREATER_THAN_SIGNIFICANCE:
			if (cm->signMatrix==NULL) {printf("EVALUATE ::: Singificance matrix is not defined!\n"); return;}
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++)
				cm->covMatrix[i][j] = (cm->signMatrix[i][j]>UPPER_SIGNIFICANCE && abs(i-j)>=SEQSEP)?1.0:0.0;
			break;
		case LESS_THAN_SIGNIFICANCE:
			if (cm->signMatrix==NULL) {printf("EVALUATE ::: Singificance matrix is not defined!\n"); return;}
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++)
				cm->covMatrix[i][j] = (cm->signMatrix[i][j]<LOWER_SIGNIFICANCE && abs(i-j)>=SEQSEP)?1.0:0.0;
			break;
		case SIGNIFICANCE_INTERVAL:
			for (i=0; i<cm->l; i++) for(j=0; j<cm->l; j++)
				cm->covMatrix[i][j] = (cm->signMatrix[i][j]>=LOWER_SIGNIFICANCE && cm->signMatrix[i][j]<=UPPER_SIGNIFICANCE && abs(i-j)>=SEQSEP)?1.0:0.0;
			break;
		case DO_NOT_EVALUATE:
		default:
			break;
	}

	return;
}

void evaluatePredictionByType(corrmut* cm, int type) {
	switch(type) {
		case GOBEL:
		case McBASC:
			evaluatePrediction(cm, GREATER_THAN_THRESHOLD);
			break;
		case OMES:
			FIRST_N = cm->l/DIVIDE;
			LOWER_SIGNIFICANCE = 0.000;
			evaluatePrediction(cm, FIRST_N_LARGEST_WITH_SIGNIFICANCE_INTERVAL);
			break;
		case ELSC:
			FIRST_N = cm->l/DIVIDE;
			evaluatePrediction(cm, FIRST_N_LARGEST);
			break;
		case SCA:
			FIRST_N = cm->l/DIVIDE;
			evaluatePrediction(cm, FIRST_N_LARGEST);
			break;
		case MI:
		case UC:
			FIRST_N = cm->l/DIVIDE;
			evaluatePrediction(cm, FIRST_N_LARGEST);
			break;
		case NO_CORRMUT:
		case RANDOM:
		default:
			break;
	}
	
		printf("::::::::::::::::::::::::>>> DIVIDE %d \n", DIVIDE);
		printf("::::::::::::::::::::::::>>> SEQSEP %d \n", SEQSEP);
		printf("::::::::::::::::::::::::>>> UPPER_SIGNIFICANCE %lf \n", UPPER_SIGNIFICANCE);
		printf("::::::::::::::::::::::::>>> LOWER_SIGNIFICANCE %lf \n", LOWER_SIGNIFICANCE);
		printf("::::::::::::::::::::::::>>> UPPER_THRESHOLD %lf \n", UPPER_THRESHOLD);
		printf("::::::::::::::::::::::::>>> LOWER_THRESHOLD %lf \n", LOWER_THRESHOLD);
	return;
}

double** calculateRocCurve(corrmut* cm, unsigned char** cnt, int type) {
	int i,j,k,N,n,m;
	int TP, FP, TN, FN;
	int step;
	int **tmp;
	double us;
	double **roc=NULL;
	double **res = NULL;
	
	if (cm->l==0 || cm->covMatrix==NULL || cnt==NULL ) {
		printf("ROC ::: Empty covariation matrix or contact map!\n");
		return NULL;
	}
	
	switch(type) {
		case GOBEL:
		case McBASC:
			// allocating memory for ROC
			roc = (double **) malloc ((100+1)*sizeof(double *));
			roc[0] = NULL;
			
			for (k=0, us=0.00; k<100; k++, us+=0.01) {
				TP = FP = TN = FN = 0;
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) if(abs(i-j)>=SEQSEP) {
					if (cm->covMatrix[i][j]>=us) {
						if (cnt[i][j]>0) TP++;
						else FP++;
					}
					else {
						if (cnt[i][j]>0) FN++;
						else TN++;
					}
				}


				roc[k] = (double *) malloc (2*sizeof(double));
				if ((TN+FP)==0 || (TP+FN)==0) {roc[k][0]=roc[k][1]=0.0; continue;}
				roc[k][0] = FP/(double)(TN+FP);
				roc[k][1] = TP/(double)(TP+FN);
				roc[k+1] = NULL;
			}
			sort(roc, 0, ASCENDING);
			break;
		case OMES:
			n = cm->l*(cm->l-1)/2.;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			sort(res, 2, DESCENDING);
			
			// allocating memory for ROC
			roc = (double **) malloc ((100*100+1)*sizeof(double *));
			
			// helper matrix
			tmp = createIntMatrix(cm->l, cm->l);
			
			step = (int)(n/100.+0.5);
			for (m=0, N=step; N<n; N+=step) for (us=0.005; us<0.5; us+=0.005) {
				TP = FP = TN = FN =0;
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) tmp[j][i] = tmp[i][j] = 0;
				for (k=0; res[k]!=NULL && k<N; k++) {
					i = (int)(res[k][0]+0.5);
					j = (int)(res[k][1]+0.5);
					if (abs(i-j)<SEQSEP) continue;
					if (cm->signMatrix[i][j]>us || cm->signMatrix[i][j]<0.0) continue;
					if (cnt[i][j]>0) TP++;
					else FP++;
					tmp[i][j] = tmp[j][i] = 1;
				}
				
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++)
					if (tmp[i][j]==0) if(abs(i-j)>=SEQSEP) {
						if (cnt[i][j]>0) FN++;
						else TN++;
					}

				if ((TN+FP)==0 || (TP+FN)==0) continue;
				roc[m] = (double *) malloc (2*sizeof(double));
				roc[m][0] = FP/(double)(TN+FP);
				roc[m][1] = TP/(double)(TP+FN);
				roc[++m] = NULL;
			}
			sort(roc, 0, ASCENDING);
			freeIntMatrix(tmp, cm->l);
			
			
			break;
		case ELSC:
		case SCA:
		case MI:
		case UC:
			n = cm->l*(cm->l-1)/2.;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			sort(res, 2, DESCENDING);
			
			// allocating memory for ROC
			roc = (double **) malloc ((100+1)*sizeof(double *));
			for (i=0; i<100; i++) roc[i] = (double *) malloc (2*sizeof(double));
			roc[i] = NULL;
			
			// helper matrix
			tmp = createIntMatrix(cm->l, cm->l);

			for (m=1; m<=100; m++) {
				N = m*n/100;
				TP = FP = TN = FN = 0;
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) tmp[j][i] = tmp[i][j] = 0;
				for (k=0; res[k]!=NULL && k<N; k++) {
					i = (int)(res[k][0]+0.5);
					j = (int)(res[k][1]+0.5);
					if (abs(i-j)<SEQSEP) continue;
					if (cnt[i][j]>0) TP++;
					else FP++;
					tmp[i][j] = tmp[j][i] = 1;
				}
				
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++)
					if (tmp[i][j]==0) if(abs(i-j)>=SEQSEP) {
						if (cnt[i][j]>0) FN++;
						else TN++;
					}

				if ((TN+FP)==0 || (TP+FN)==0) {roc[m-1][0]=roc[m-1][1]=0.0; continue;}
				roc[m-1][0] = FP/(double)(TN+FP);
				roc[m-1][1] = TP/(double)(TP+FN);
			}
			sort(roc, 0, ASCENDING);
			freeIntMatrix(tmp, cm->l);
			break;
		case RANDOM:
			n = 0;
			for (i=0; i<cm->l; i++) for(j=0; j<i; j++) if (cm->covMatrix[i][j]>0.0) n++;
			res = (double**) malloc ((n+1)*sizeof(double*));
			res[0] = NULL;
			
			for (k=0, i=0; i<cm->l; i++) for(j=0; j<i; j++) if (cm->covMatrix[i][j]>0.0) {
				res[k] = (double*) malloc (3*sizeof(double));
				res[k][0] = (double)i;
				res[k][1] = (double)j;
				res[k][2] = cm->covMatrix[i][j];
				res[++k] = NULL;
			}
			
			shuffle(res);
			
			// allocating memory for ROC
			roc = (double **) malloc ((100+1)*sizeof(double *));
			for (i=0; i<100; i++) roc[i] = (double *) malloc (2*sizeof(double));
			roc[i] = NULL;
			
			// helper matrix
			tmp = createIntMatrix(cm->l, cm->l);

			for (m=1; m<=100; m++) {
				N = m*n/100;
				TP = FP = TN = FN = 0;
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++) tmp[i][j] = 0;
				for (k=0; res[k]!=NULL && k<N; k++) {
					i = (int)(res[k][0]+0.5);
					j = (int)(res[k][1]+0.5);
					if (abs(i-j)<SEQSEP) continue;
					if (cnt[i][j]>0) TP++;
					else FP++;
					tmp[i][j] = tmp[j][i] = 1;
				}
				
				for (i=0; i<cm->l; i++) for (j=0; j<=i; j++)
					if (tmp[i][j]==0) if(abs(i-j)>=SEQSEP) {
						if (cnt[i][j]>0) FN++;
						else TN++;
					}

				if ((TN+FP)==0 || (TP+FN)==0) {roc[m-1][0]=roc[m-1][1]=0.0; continue;}
				roc[m-1][0] = FP/(double)(TN+FP);
				roc[m-1][1] = TP/(double)(TP+FN);
			}
			sort(roc, 0, ASCENDING);
			freeIntMatrix(tmp, cm->l);
			break;
		case NO_CORRMUT:
		default:
			break;
	}
	return roc;
}



void sort (double **what, int by, int how) {
	int i,j;
	double *t;
	
	if (what==NULL) return;
	if (what[0]==NULL) return;
	
	for (i=0; what[i]!=NULL; i++) for (j=0; j<i; j++)
		if ( how*what[i][by] < how*what[j][by] ) {
			t = what[i];
			what[i] = what[j];
			what[j] = t;
		}
	return;
}

void shuffle(double** what) {
	int i,j,n;
	double *t;
	gsl_rng * r;

	if (what==NULL) return;
	if (what[0]==NULL) return;
	
	gsl_rng_env_setup();
	r = gsl_rng_alloc (gsl_rng_default);

	for (n=0; what[n]!=NULL; n++);
	if (n<2) return;
	for (i=0; i<n; i++) {
		j = (int)(n*gsl_rng_uniform(r));
		t = what[i];
		what[i] = what[j];
		what[j] = t;
	}
	return;
}


void calculateMsaSimMatrix(corrmut* cm) {
	int i,j,l;
	int eq, neq;
	PRECISION sum = 0;

	cm->msaSimMatrix = createPRECISIONMatrix(cm->n, cm->n);
	for (i=0; i<cm->n; i++) {
		cm->msaSimMatrix[i][i] = 0.0;
		for (j=0; j<i; j++) {
			eq = 0;
			neq = 0;
			for (l=0; l<cm->l; l++) (cm->msa[i][l]==cm->msa[j][l])?eq++:neq++;
			sum += cm->msaSimMatrix[i][j] = (PRECISION)neq/(PRECISION)eq;
		}
	}
	for (i=0; i<cm->n; i++) for (j=0; j<i; j++) cm->msaSimMatrix[j][i] = (cm->msaSimMatrix[i][j] /= sum);

	return;
}

void reduceMsaMatrixValues(corrmut* cm) {
	int i,j,l;
	PRECISION N;
	
	N = (PRECISION)(cm->n*(cm->n-1)/2.-1.0);
	for (l=0; l<cm->l; l++) for (i=0; i<cm->n; i++) for (j=0; j<i; j++) {
		if (cm->msaMatrixStddev[l] == 0.0) {
			cm->msaMatrix[l][j][i] = cm->msaMatrix[l][i][j] = 0.0;
			continue;
		}
		cm->msaMatrix[l][i][j] -= cm->msaMatrixMean[l];				// mean of the matrix
		cm->msaMatrix[l][i][j] /= cm->msaMatrixStddev[l];			// stddev of the matrix
//		cm->msaMatrix[l][i][j] *= sqrt(cm->msaSimMatrix[i][j]);	/		// similarity of MSA sequences
//		cm->msaMatrix[l][i][j] /= N;						// number of MSA sequences
		cm->msaMatrix[l][j][i] = cm->msaMatrix[l][i][j];			// set m[j][i] = m[i][j]
	}
	return;
}

void calculateMsaMatrixValues(corrmut* cm) {
	int i,j,l;
	PRECISION N;

	cm->msaMatrixMean = (PRECISION*) malloc (cm->l*sizeof(PRECISION));
	cm->msaMatrixStddev = (PRECISION*) malloc (cm->l*sizeof(PRECISION));
	N = (PRECISION)(cm->n*(cm->n-1)/2.);
	for (l=0; l<cm->l; l++) {
		cm->msaMatrixMean[l] = 0.0;
		cm->msaMatrixStddev[l] = 0.0;
		for (i=0; i<cm->n; i++) for (j=0; j<i; j++) {
				cm->msaMatrixMean[l] += cm->msaMatrix[l][i][j];
				cm->msaMatrixStddev[l] += cm->msaMatrix[l][i][j]*cm->msaMatrix[l][i][j];
		}

		//mean = Sum/n
		//stdev = sqrt((Sum_sqr - Sum*Sum/n)/(n - 1))
		cm->msaMatrixStddev[l] = sqrt(fabs((cm->msaMatrixStddev[l]-cm->msaMatrixMean[l]*cm->msaMatrixMean[l]/N)/(N-1.0)));
		cm->msaMatrixMean[l] /= N;
	}
	return;
}

void calculateConservation(corrmut* cm, int type) {
	int i,j,l;
	PRECISION s;
	PRECISION* tmp;

	if (cm->n<=1) return;
	cm->cons = (PRECISION*) malloc (cm->l*sizeof(PRECISION));
	switch (type) {
		case LIN:
			break;
		case QUAD:
			s = cm->n*(cm->n-1)/2.;
			for (l=0; l<cm->l; l++) {
				cm->cons[l] = 0.0;
				for (i=0; i<cm->n-1; i++) for (j=i+1; j<cm->n; j++)
					if (cm->msa[i][l] == cm->msa[j][l]) cm->cons[l]++;
				cm->cons[l] /= s;
			}
			break;
		case ENTROPY:
		case LOG:
			if (cm->probI==NULL) break;
			for (l=0; l<cm->l; l++) {
				cm->cons[l] = 0.0;
				for (i=0; i<AA_NUM; i++) if (cm->probI[l][i]!=0.0) cm->cons[l] -= cm->probI[l][i]*log(cm->probI[l][i])/*log(2.)*/;
			}
			break;
		case ENTROPY_F:
			/* TODO */
			/* lehet, hogy ez felesleges... */
			tmp = (PRECISION*) malloc (AA_NUM*sizeof(PRECISION));
			for (l=0; l<cm->l; l++) {
				cm->cons[l] = 0.0;
				for (i=0; i<AA_NUM; i++) tmp[i] = 0.;
				for (i=0; i<AA_NUM; i++) tmp[AA_TYPE[i]] = cm->probI[l][i];
				for (i=0; i<AA_NUM; i++) if (tmp[i]!=0.0) cm->cons[l] -= tmp[i]*log(tmp[i])/*log(2.)*/;
			}
			free(tmp);
			break;			
		case DG_STAT:
			if (cm->probI==NULL) break;
			for (l=0; l<cm->l; l++) {
				cm->cons[l] = 0.0;
				for (i=0; i<AA_NUM; i++) cm->cons[l] += pow(log(cm->probI[l][i]/cm->probAA[i]), 2.);
				cm->cons[l] = /* multiply by kT* */sqrt(cm->cons[l]);
			}
			break;
		default:
			calculateConservation(cm, QUAD);
			break;
	}
	return;
}

void calculateProbability(corrmut* cm, int type) {
	int i,j,k,l,n;
	
	switch (type) {
		case PROB_I:
			// calculate AA probability at column I
			cm->probI = createPRECISIONMatrix(cm->l, AA_NUM);
			for (l=0; l<cm->l; l++) for (i=0; i<AA_NUM; i++) cm->probI[l][i] = 0.0;
			for (l=0; l<cm->l; l++) {
				for (i=0; i<cm->n; i++) cm->probI[l][AA_INDEX(cm->msa[i][l])]++;
				for (i=0; i<AA_NUM; i++) cm->probI[l][i] /= (PRECISION)cm->n;
			}
			break;

		case PROB_I_F:
			// calculate reduced type AA probability at column I
			cm->probI = createPRECISIONMatrix(cm->l, AA_NUM);
			for (l=0; l<cm->l; l++) for (i=0; i<AA_NUM; i++) cm->probI[l][i] = 0.0;
			for (l=0; l<cm->l; l++) {
				for (i=0; i<cm->n; i++) cm->probI[l][AA_GROUPS[AA_INDEX(cm->msa[i][l])]]++;
				for (i=0; i<AA_NUM; i++) cm->probI[l][i] /= (PRECISION)cm->n;
			}
			break;

		case PROB_IJ:
			// calculate AA pair probability at column I-J
			// TODO speedup with l-k-i-j vs k-l-j-i swapping
			cm->probIJ = (PRECISION****) malloc (cm->l*sizeof(PRECISION***));
			for (i=0; i<cm->l; i++) {
				cm->probIJ[i] = (PRECISION***) malloc (cm->l*sizeof(PRECISION**));
				for (j=0; j<cm->l; j++) {
					cm->probIJ[i][j] = createPRECISIONMatrix(AA_NUM,AA_NUM);
					for (k=0; k<AA_NUM; k++) for (l=k; l<AA_NUM; l++) cm->probIJ[i][j][l][k] = cm->probIJ[i][j][k][l] = 0.0;
					for (k=0; k<cm->n; k++)
						cm->probIJ[i][j][AA_INDEX(cm->msa[k][i])][AA_INDEX(cm->msa[k][j])]++;
					for (k=0; k<AA_NUM; k++) for (l=0; l<AA_NUM; l++) cm->probIJ[i][j][k][l] /= (PRECISION)cm->n;
				}
			}
			break;

		case PROB_IJ_F:
			// calculate reduced type AA probability at column I-J
			cm->probIJ = (PRECISION****) malloc (cm->l*sizeof(PRECISION***));
			for (i=0; i<cm->l; i++) {
				cm->probIJ[i] = (PRECISION***) malloc (cm->l*sizeof(PRECISION**));
				for (j=0; j<cm->l; j++) {
					cm->probIJ[i][j] = createPRECISIONMatrix(AA_NUM,AA_NUM);
					for (k=0; k<AA_NUM; k++) for (l=k; l<AA_NUM; l++) cm->probIJ[i][j][l][k] = cm->probIJ[i][j][k][l] = 0.0;
					for (k=0; k<cm->n; k++)
						cm->probIJ[i][j][AA_GROUPS[AA_INDEX(cm->msa[k][i])]][AA_GROUPS[AA_INDEX(cm->msa[k][j]-64)]]++;
					for (k=0; k<AA_NUM; k++) for (l=0; l<AA_NUM; l++) cm->probIJ[i][j][k][l] /= (PRECISION)cm->n;
				}
			}
			break;

		case PROB_AA:

			/* TODO */
			/* gapek szamolasa hogy legyen ??? */
			// calculate global AA probability in the given MSA
			cm->probAA = (PRECISION*) malloc (AA_NUM*sizeof(PRECISION));
			for (i=0; i<AA_NUM; i++) cm->probAA[i] = 0.0;
			n = 0;
			for (k=0; k<cm->n; k++) for (i=0; i<cm->l; i++) {
//				if (cm->msa[k][i]=='-') continue;
				cm->probAA[AA_INDEX(cm->msa[k][i])]++;
				n++;
			}
			for (i=0; i<AA_NUM; i++) cm->probAA[i] /= (PRECISION)n;
			break;

		default:
			calculateProbability(cm, PROB_I);
			break;
	}

	return;
}

void printMsa(corrmut* cm) {
        int i;

        if (cm==NULL) {printf("CorrMut struct is not defined!\n"); return;}
        if (cm->msa==NULL) {printf("CorrMut->MSA is not defined!\n"); return;}
        for (i=0; i<cm->n; i++) printf("%s\n", cm->msa[i]);
        return;
}

void printCovarianceMatrix(FILE* f, corrmut* cm) {
        int i,j;
        
        if (cm->covMatrix==NULL) return;
        for (i=0; i<cm->l; i++) {
                for (j=0; j<cm->l; j++) fprintf(f, "%+6.5f ", cm->covMatrix[i][j]);
                fprintf(f, "\n");
        }
        return;
}

void printCovarianceMatrixAsList(FILE* f, corrmut* cm, char* format) {
        int i,j;
        
        if (cm->covMatrix==NULL) return;
	fprintf(f, "i\tj\tscore\n");
        for (i=0; i<cm->l; i++)
                for (j=i+1; j<cm->l; j++) {
			fprintf(f, "%d\t%d\t", i, j);
			fprintf(f, (const char*)format, cm->covMatrix[i][j]);
			fprintf(f, "\n");
		}
        return;
}

void printSignificanceMatrix(FILE* f, corrmut* cm) {
        int i,j;
        
        if (cm->signMatrix==NULL) return;
        for (i=0; i<cm->l; i++) {
                for (j=0; j<cm->l; j++) fprintf(f, "%+6.5f ", cm->signMatrix[i][j]);
                fprintf(f, "\n");
        }
        return;
}

void printCovarianceMatrixWithSignificance(FILE* f, corrmut* cm) {
        int i,j;
        
        if (cm->covMatrix==NULL) return;
        if (cm->signMatrix==NULL) return;
        for (i=0; i<cm->l-1; i++) if (cm->probI[i][AA_NUM-1]<=GAP_THRESHOLD) for (j=i+1; j<cm->l; j++) if (cm->probI[j][AA_NUM-1]<=GAP_THRESHOLD)
                if (cm->signMatrix[i][j] >= 0.0)
                        fprintf(f, "%d\t%d\t%.25lf\t%.2lf\n", i+1, j+1, cm->signMatrix[i][j], cm->covMatrix[i][j]);
        return;
}

void plotCovarianceMatrix(char * filename) {
        int i;
        char * cmd = (char *) malloc (512*sizeof(char));

        strcpy(cmd, "gnuplot -persist -e \'reset; set view map; set size ratio 1; splot \"");
        strcat(cmd, filename);
        strcat(cmd, "\" matrix with image\'");
        printf("%s\n", cmd);
        i = system(cmd);
        free(cmd);
        return;
}

void printConservation(corrmut* cm) {
        int l;

        if (cm->cons==NULL) return;
        for (l=0; l<cm->l; l++) printf("%d %lf\n", l, cm->cons[l]);
        return;
}

void printAllProbabilities(corrmut* cm) {
	int i;

	for (i=0; i<PROB_TYPE_END; i++) printProbability(cm, i);
	return;
}

void printProbability(corrmut* cm, int type) {
	int i,j,k,l;

	switch (type) {
		case PROB_I:
		case PROB_I_F:
			if (cm->probI == NULL) break;
			for (i=0; i<AA_NUM; i++) printf("    %c    ", i+64);
			printf("\n");
			for (l=0; l<cm->l; l++) {
				for (i=0; i<AA_NUM; i++) printf("%lf ", cm->probI[l][i]);
				printf("\n");
			}
			break;

		case PROB_IJ:
		case PROB_IJ_F:
			if (cm->probIJ==NULL) break;
			for (i=0; i<cm->l; i++) for (j=0; j<cm->l; j++) {
				printf("< %d, %d>\n", i,j);
				for (k=0; k<AA_NUM; k++) {
					for (l=0; l<AA_NUM; l++) printf("%lf ", cm->probIJ[i][j][k][l]);
					printf("\n");
				}
				printf("//\n\n");
			}
			break;

		case PROB_AA:
			if (cm->probAA==NULL) break;
			for (i=0; i<AA_NUM; i++) printf("    %c    ", i+64);
			printf("\n");
			for (i=0; i<AA_NUM; i++) printf("%lf ", cm->probAA[i]);
			printf("\n");
			break;

		default:
			break;
	}

	return;
}

void loadMsaFromFile(corrmut* cm, char* filename) {
        if (cm==NULL) {
                printf("CorrMut struct is not defined!\n");
                return;
        }
        cm->msa = getSequences(getNamesSequences(readLinesFromFile(filename)));
	if (cm->msa==NULL) {printf("CORRMUT ::: Empty multiple sequence alignment!\n"); return;}
        cm->l = strlen(cm->msa[0]);
        for (cm->n=0; cm->msa[cm->n]!=NULL; cm->n++) if (strlen(cm->msa[cm->n]) != cm->l) printf("Error MSA length\n");
        return;
}

void createMatrixFromMSA(corrmut* cm) {
	int i,j,l;

	cm->msaMatrix = (PRECISION***) malloc ((cm->l)*sizeof(PRECISION**));
	for (l=0; l<cm->l; l++) {
		cm->msaMatrix[l] = createPRECISIONMatrix(cm->n, cm->n);
		for (i=0; i<cm->n; i++) {
			cm->msaMatrix[l][i][i] = 0.0;
			for (j=0; j<i; j++) cm->msaMatrix[l][j][i] = cm->msaMatrix[l][i][j] = (PRECISION) BLOSUM62[AA_INDEX(cm->msa[i][l])][AA_INDEX(cm->msa[j][l])];
		}
	}
	return;
}

void freeCorrmut(corrmut* cm) {
	int i,j;

	if (cm->msa!=NULL) {
		for (i=0; i<cm->n; i++) if (cm->msa[i]!=NULL) free(cm->msa[i]);
		free(cm->msa);
	}
	if (cm->cons!=NULL) free(cm->cons);
	if (cm->probAA!=NULL) free(cm->probAA);
	if (cm->probI!=NULL) freePRECISIONMatrix(cm->probI, cm->l);

	if (cm->probIJ!=NULL) {
		for (i=0; i<cm->l; i++) {
			for(j=0; j<cm->l; j++) freePRECISIONMatrix(cm->probIJ[i][j], AA_NUM);
			free(cm->probIJ[i]);
		}
		free(cm->probIJ);
	}
	if (cm->msaMatrix!=NULL) {
		for (i=0; i<cm->l; i++)	freePRECISIONMatrix(cm->msaMatrix[i], cm->n);
		free(cm->msaMatrix);
	}
	if (cm->msaMatrixMean!=NULL) free(cm->msaMatrixMean);
	if (cm->msaMatrixStddev!=NULL) free(cm->msaMatrixStddev);
	if (cm->msaSimMatrix!=NULL) freePRECISIONMatrix(cm->msaSimMatrix, cm->n);
	if (cm->covMatrix!=NULL) freePRECISIONMatrix(cm->covMatrix, cm->l);
	if (cm->signMatrix!=NULL) freePRECISIONMatrix(cm->signMatrix, cm->l);
	if (cm!=NULL) free(cm);
	cm = NULL;
	return;
}

