/*  corrmut.h
 *
 *  Copyright (C) 2012, Institute of Enzymology,
 *  Daniel Kozma,  kozmad at enzim dot hu

 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**********************************************************
  A utility for calculating multiple correlation on MSA 
 to find correlated mutations.                         

  USAGE:
    cm = createNewCorrmut();
    loadMsaFromFile(cm, filename)
    calculateCovarianceMatrix(cm, METHOD_TYPE);
    printCovarianceMatrix(cm);

**********************************************************/
#ifndef __corrmut_h_
#define __corrmut_h_


#ifdef __cplusplus
extern "C" {
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_randist.h>
#include "nonstdio.h"
#include "matrix.h"
#include "aminoacids.h"



//#if __WORDSIZE == 32
//#define PRECISION float
//#define createPRECISIONMatrix createFloatMatrix
//#define freePRECISIONMatrix freeFloatMatrix
//#elif __WORDSIZE == 64
#define PRECISION double
#define createPRECISIONMatrix createDoubleMatrix
#define freePRECISIONMatrix freeDoubleMatrix
//#endif

#define ASCENDING  +1.
#define DESCENDING -1.


/* TODO */
/* test the following methods: 
	GOBEL		99% tested
	OMES    	99% tested
	SCA		99% tested
	MI		99% tested
	ELSC		80% tested
*/

enum CORRMUT_TYPE {NO_CORRMUT, RANDOM, GOBEL, McBASC, OMES, ELSC, /*ELSC_PLUS,*/ SCA, MI, UC, CORRMUT_TYPE_END};
enum CONS_TYPE {LIN, QUAD, LOG, ENTROPY, ENTROPY_F, DG_STAT, CONS_TYPE_END};
enum PROB_TYPE {PROB_I, PROB_IJ, PROB_I_F, PROB_IJ_F, PROB_AA, PROB_TYPE_END};
enum EVAL_TYPE {FIRST_N_LARGEST, FIRST_N_SMALLEST, FIRST_N_LARGEST_WITH_SIGNIFICANCE_INTERVAL, GREATER_THAN_THRESHOLD, LESS_THAN_THRESHOLD, THRESHOLD_INTERVAL, GREATER_THAN_SIGNIFICANCE, LESS_THAN_SIGNIFICANCE, SIGNIFICANCE_INTERVAL, DO_NOT_EVALUATE, EVAL_TYPE_END};

static const char *corrmut_type[] = {"NO_CORRMUT", "RANDOM", "GOBEL", "McBASC", "OMES", "ELSC", /*"ELSC_PLUS",*/ "SCA", "MI", "UC"};
static const char *cons_type[] = {"LIN", "QUAD", "LOG", "ENTROPY", "ENTROPY_F", "DG_STAT"};
static const char *prob_type[] = {"PROB_I", "PROB_IJ", "PROB_I_F", "PROB_IJ_F", "PROB_AA"};
static const char *eval_type[] = {"FIRST_N_LARGEST", "FIRST_N_SMALLEST", "FIRST_N_LARGEST_WITH_SIGNIFICANCE_INTERVAL", "GREATER_THAN_THRESHOLD", "LESS_THAN_THRESHOLD", "THRESHOLD_INTERVAL", "GREATER_THAN_SIGNIFICANCE", "LESS_THAN_SIGNIFICANCE, SIGNIFICANCE_INTERVAL", "DO_NOT_EVALUATE"};

int N_CONTACTS;
int SEQSEP;
int DIVIDE;
int FIRST_N;
double UPPER_THRESHOLD;
double LOWER_THRESHOLD;
double UPPER_SIGNIFICANCE;
double LOWER_SIGNIFICANCE;
char *ALPHABET;
int *AA_GROUPS;

#define GAP_THRESHOLD 0.1

struct _corrmut {
	int n;
	int l;
	char ** msa;
	PRECISION* cons;
	PRECISION* probAA;
	PRECISION** probI;
	PRECISION**** probIJ;
	PRECISION*** msaMatrix;
	PRECISION* msaMatrixMean;
	PRECISION* msaMatrixStddev;
	PRECISION** msaSimMatrix;
	PRECISION** covMatrix;
	PRECISION** signMatrix;
};
typedef struct _corrmut corrmut;


corrmut* createNewCorrmut(void);
void createMatrixFromMSA(corrmut* cm);
void loadMsaFromFile(corrmut* cm, char * filename);
void freeCorrmut(corrmut* cm);

void calculateCovarianceMatrixGOBEL(corrmut* cm);			// simple co-variance calculation
void calculateCovarianceMatrixOMES(corrmut* cm);			// Observed Minus Expected Squared
void calculateCovarianceMatrixELSC(corrmut* cm);			// Explicit Likelihood of Subset Co-variation
void calculateCovarianceMatrixSCA(corrmut* cm);				// Statistical Coupling Analysis
void calculateCovarianceMatrixMI(corrmut* cm);				// Mutual Information
void calculateCovarianceMatrixUC(corrmut* cm);				// Ucertanity Coefficient
void calculateCovarianceMatrix(corrmut* cm, int type);

void calculateConservation(corrmut* cm, int type);
void calculateProbability(corrmut* cm, int type);
void calculateMsaSimMatrix(corrmut* cm);
void reduceMsaMatrixValues(corrmut* cm);
void calculateMsaMatrixValues(corrmut* cm);
void calculateNthOrderCovariance(corrmut *cm);

void randomPrediction(corrmut* cm);
void evaluatePrediction(corrmut* cm, int type);
void evaluatePredictionByType(corrmut* cm, int type);
double** calculateRocCurve(corrmut* cm, unsigned char **cnt, int type);

void printMsa(corrmut* cm);
void printCovarianceMatrix(FILE* f, corrmut* cm);
void printCovarianceMatrixAsList(FILE* f, corrmut* cm, char* format);
void printSignificanceMatrix(FILE* f, corrmut* cm);
void printConservation(corrmut* cm);
void printProbability(corrmut* cm, int type);
void printAllProbabilities(corrmut* cm);

void plotCovarianceMatrix(char * filename);

void sort(double **what, int by, int how);
void shuffle(double **what);


#ifdef __cplusplus
}
#endif

#endif /*corrmut.h*/





